@@ -25,6 +25,5 @@ formatter = FuncFormatter(millions)
 ax = subplot(111)
 ax.yaxis.set_major_formatter(formatter)
 bar(x, money)
-ax.set_xticks( x + 0.5)
-ax.set_xticklabels( ('Bill', 'Fred', 'Mary', 'Sue') )
+xticks( x + 0.5,  ('Bill', 'Fred', 'Mary', 'Sue') )
 show()
