@@ -9,11 +9,11 @@ g = 2*e
 h = 2*f
 
 figure(1)  
-#errorbar(t, s, e, fmt='o')             # vertical symmetric
+errorbar(t, s, e, fmt='o')             # vertical symmetric
 #errorbar(t, s, None, f, fmt='o')       # horizontal symmetric
 #errorbar(t, s, e, f, fmt='o')          # both symmetric
 #errorbar(t, s, [e,g], [f,h], fmt='o')  # both asymmetric
-errorbar(t, s, [e,g], f, fmt='o', ecolor='g')      # both mixed
+#errorbar(t, s, [e,g], f, fmt='o', ecolor='g')      # both mixed
 #errorbar(t, s, e, [f,h], fmt='o')      # both mixed
 #errorbar(t, s, [e,g], fmt='o')         # vertical asymmetric
 #errorbar(t, s, yerr=e, fmt='o')        # named
